import petl
from gargantools.utils.utils import check_columnName, check_date, check_fileEncoding, check_fileExtension, check_statusCell, check_unacceptedCharactersQuote, check_unacceptedCharactersTab, get_fileContent, get_fileDelimiter


CONTEXTS_COLUMN_NAMES = ["Publication Day", "Publication Month", "Publication Year", "Authors", "Title", "Abstract", "Source"]
TERMS_COLUMN_NAMES = ["status", "label"]


def contexts_check_fileContent(fileContent):
    header = ("Publication Day", "Publication Month", "Publication Year", "Authors", "Title", "Abstract", "Source")
    constraints = [
        dict(
            name = "pub_day", field = "Publication Day", test = int, assertion = lambda x: 0 < x <= 31
        ),
        dict(
            name = "pub_month", field = "Publication Month", test = int, assertion = lambda x : 0 < x <= 12
        ),
        dict(
            name = "pub_year", field = "Publication Year", test = int, assertion = lambda x: x > 0
        ),
        dict(
            name = "date_format", field = ["Publication Day", "Publication Month", "Publication Year"], assertion = lambda row : check_date(row[2], row[1], row[0])
        ),
        dict(
            name = "str_formatQuote", field = "Authors", assertion = lambda x : check_unacceptedCharactersQuote(x)
        ),
        dict(
            name = "str_formatTab", field = "Authors", assertion = lambda x : check_unacceptedCharactersTab(x)
        ),
        dict(
            name = "str_formatQuote", field = "Title", assertion = lambda x : check_unacceptedCharactersQuote(x)
        ),
        dict(
            name = "str_formatTab", field = "Title", assertion = lambda x : check_unacceptedCharactersTab(x)
        ),
        dict(
            name = "str_formatQuote", field = "Abstract", assertion = lambda x : check_unacceptedCharactersQuote(x)
        ),
        dict(
            name = "str_formatTab", field = "Abstract", assertion = lambda x : check_unacceptedCharactersTab(x)
        ),
        dict(
            name = "str_formatQuote", field = "Source", assertion = lambda x : check_unacceptedCharactersQuote(x)
        ),
        dict(
            name = "str_formatTab", field = "Source", assertion = lambda x : check_unacceptedCharactersTab(x)
        )
    ]
    
    dataTable = fileContent.values.tolist()
    dataTable.insert(0, fileContent.columns.to_list())

    problemCells = petl.validate(dataTable, constraints, header)

    if problemCells.len() > 1:
        return False, problemCells
    else:
        return True, None

def terms_check_fileContent(fileContent):
    header = ("status", "label")
    constraints = [
        dict(
            name = "acceptedVariable", field = "status", assertion = lambda x : check_statusCell(x)
        ),
        dict(
            name = "str_formatQuote", field = "label", assertion = lambda x : check_unacceptedCharactersQuote(x)
        ),
        dict(
            name = "str_formatTab", field = "label", assertion = lambda x : check_unacceptedCharactersTab(x)
        )
    ]
    
    dataTable = fileContent.values.tolist()
    dataTable.insert(0, fileContent.columns.to_list())

    problemCells = petl.validate(dataTable, constraints, header)

    if problemCells.len() > 1:
        return False, problemCells
    else:
        return True, None

def validation(request, cat_type):
    if 'file' not in request.files:
        return "Bad request, missing file\n", 400
    else:
        file = request.files['file']
        if not (file and check_fileExtension(file.filename, {"csv", "tsv"})):
            return "Incorrect file format or file format not found\n", 400
        else:
            file_encoding = check_fileEncoding(file)
            if file_encoding is None:
                return "Could not read the file\n", 422
            elif get_fileDelimiter(file, file_encoding) != '\t':
                return "Error while getting file delimiter.\n Can be the following : \n - File delimiter not found \n - Incorrect file delimiter, should be a tabulation \n - File is malformed\n", 422
            else:
                file_content = get_fileContent(file, '\t')
                if file_content is None:
                    print('here')
                    return "Could not read the file\n", 422
                else:
                    header = file_content.keys()
                    if cat_type == "contexts":
                        state, notFoundColumn = check_columnName(header, CONTEXTS_COLUMN_NAMES)
                    elif cat_type == "terms":
                        state, notFoundColumn = check_columnName(header, TERMS_COLUMN_NAMES)
                    if not state:
                        return f"Some column names were not found {','.join(notFoundColumn)}\n", 422
                    else:
                        if cat_type == "contexts":
                            state, problems = contexts_check_fileContent(file_content)
                        elif cat_type == "terms":
                            state, problems = terms_check_fileContent(file_content)
                        if state:
                            return "Correct file\n", 200
                        else:
                            if problems is not None:
                                return 'Incorrect file - File is not compatible with GarganText\n', 422
                            else:
                                return 'Unexpected error in file\n', 422
                            