from io import BytesIO
from itertools import product
from werkzeug.datastructures import FileStorage

import pytest

from tests.conftest import id_files


@pytest.mark.parametrize("cat_type", ["contexts", "terms"])
def test_missingFile(client, cat_type):
    if cat_type == "contexts":
        response = client.post('/contexts')
    elif cat_type == "terms":
        response = client.post('/terms')
    
    assert response.data == b"Bad request, missing file\n"
    assert response.status_code == 400

@pytest.mark.parametrize("test_file_copy", product(["contexts", "terms"], ["incorrect.csv", "malformed.csv", "incorrectDelimiter.csv", "incorrectHeader.csv", "incorrectExtension"]), indirect=True, ids=lambda param : id_files(param))
def test_incorrectFile(client, test_file_copy):
    test_file, cat_type = test_file_copy
    with open(test_file, "rb") as f:
        f_data = BytesIO(f.read())
        file_storage = FileStorage(f_data, filename=test_file)
        if cat_type == "contexts":
            response = client.post('/contexts', data={'file': file_storage}, content_type='multipart/form-data')
        elif cat_type == "terms":
            response = client.post('/terms', data={'file': file_storage}, content_type='multipart/form-data')

        print(test_file_copy)
        print(response.data)

        if "incorrect.csv" in test_file:
            assert response.data == b'Incorrect file - File is not compatible with GarganText\n'
        elif "malformed.csv" in test_file:
            assert response.data == b"Could not read the file\n" or response.data == b"Error while getting file delimiter.\n Can be the following : \n - File delimiter not found \n - Incorrect file delimiter, should be a tabulation \n - File is malformed\n"
        elif "incorrectDelimiter.csv" in test_file:
            assert response.data == b"Error while getting file delimiter.\n Can be the following : \n - File delimiter not found \n - Incorrect file delimiter, should be a tabulation \n - File is malformed\n"
        elif "incorrectHeader.csv" in test_file:
            assert b"Some column names were not found" in response.data
        elif "incorrectExtension" in test_file:
            assert response.data == b"Incorrect file format or file format not found\n"
            assert response.status_code == 400
        else:
            assert False

        if "incorrectExtension" not in test_file:
            assert response.status_code == 422

@pytest.mark.parametrize("test_file_copy", product(["contexts", "terms"], ["correct.csv"]), indirect=True, ids=lambda param : id_files(param))
def test_correctFile(client, test_file_copy):
    test_file, cat_type = test_file_copy
    with open(test_file, "rb") as f:
        f_data = BytesIO(f.read())
        file_storage = FileStorage(f_data, filename=test_file)
        if cat_type == "contexts":
            response = client.post('/contexts', data={'file': file_storage}, content_type='multipart/form-data')
        elif cat_type == "terms":
            response = client.post('/terms', data={'file': file_storage}, content_type='multipart/form-data')

        assert response.data == b"Correct file\n"
        assert response.status_code == 200
